import React, { forwardRef, useEffect, useImperativeHandle } from 'react';
import { fabric } from 'fabric';

const createPolygon = ({
  xStart,
  yStart,
  type,
  size
}) => {
  const points = [];

  for (let i = 1; i <= type; i += 1) {
    points.push({
      x: xStart + size * Math.sin((i * 2 * Math.PI - Math.PI) / type),
      y: yStart + size * Math.cos((i * 2 * Math.PI - Math.PI) / type)
    });
  }

  return points;
};
const Polygon = forwardRef(({
  canvasProps,
  children
}, ref) => {
  const [canvas, setCanvas] = React.useState(null);
  const [drawMode, setDrawMode] = React.useState(false);
  const id = (new Date().getTime() + Math.random() * 10000).toString();
  let lineArray = [];
  let pointArray = [];
  let activeLine;
  let activeShape;
  document.addEventListener("keydown", event => {
    if (event.keyCode === 46) {
      handleDeletePolygon();
    }

    if (event.keyCode === 78) {
      handleNextPolygon();
    }

    if (event.keyCode === 80) {
      handlePreviousPolygon();
    }
  });

  const polygonPositionHandler = (index, _dim, _finalMatrix, fabricObject) => {
    const x = fabricObject.points[index].x - fabricObject.pathOffset.x;
    const y = fabricObject.points[index].y - fabricObject.pathOffset.y;
    return fabric.util.transformPoint({
      x,
      y
    }, fabric.util.multiplyTransformMatrices(fabricObject.canvas.viewportTransform, fabricObject.calcTransformMatrix()));
  };

  const actionHandler = (_eventData, transform, x, y) => {
    const polygon = transform.target;
    const currentControl = polygon.controls[polygon.__corner];
    const mouseLocalPosition = polygon.toLocalPoint(new fabric.Point(x, y), "center", "center");

    const polygonBaseSize = polygon._getNonTransformedDimensions();

    const size = polygon._getTransformedDimensions(0, 0);

    const xPoint = mouseLocalPosition.x * polygonBaseSize.x / size.x + polygon.pathOffset.x;
    const yPoint = mouseLocalPosition.y * polygonBaseSize.y / size.y + polygon.pathOffset.y;
    const xOCoordsPoint = polygon.oCoords[`p${currentControl.pointIndex}`].x;
    const yOCoordsPoint = polygon.oCoords[`p${currentControl.pointIndex}`].y;

    if (xOCoordsPoint > canvas.width || xOCoordsPoint < 0 || yOCoordsPoint > canvas.height || yOCoordsPoint < 0) {
      return false;
    }

    polygon.points[currentControl.pointIndex] = {
      x: xPoint,
      y: yPoint
    };
    return true;
  };

  const anchorWrapper = (anchorIndex, fn) => function (eventData, transform, x, y) {
    const fabricObject = transform.target;
    const absolutePoint = fabric.util.transformPoint({
      x: fabricObject.points[anchorIndex].x - fabricObject.pathOffset.x,
      y: fabricObject.points[anchorIndex].y - fabricObject.pathOffset.y
    }, fabricObject.calcTransformMatrix());
    const actionPerformed = fn(eventData, transform, x, y);

    fabricObject._setPositionDimensions({});

    const polygonBaseSize = fabricObject._getNonTransformedDimensions();

    const newX = (fabricObject.points[anchorIndex].x - fabricObject.pathOffset.x) / polygonBaseSize.x;
    const newY = (fabricObject.points[anchorIndex].y - fabricObject.pathOffset.y) / polygonBaseSize.y;
    fabricObject.setPositionByOrigin(absolutePoint, newX + 0.5, newY + 0.5);
    return actionPerformed;
  };

  const resizePolygon = evt => {
    const poly = evt.target;
    poly.edit = false;
    poly.objectCaching = false;
    poly.controls = fabric.Object.prototype.controls;
    poly.cornerStyle = "rect";
    poly.hasBorders = true;
    canvas.requestRenderAll();
  };

  const editPolygon = evt => {
    const poly = evt.target;
    const lastControl = poly.points.length - 1;
    poly.edit = true;
    poly.objectCaching = false;
    poly.cornerStyle = "circle";
    poly.cornerColor = "rgba(0,0,255,0.5)";
    poly.controls = poly.points.reduce((acc, _point, index) => {
      acc[`p${index}`] = new fabric.Control({
        positionHandler: (dim, finalMatrix, fabricObject) => polygonPositionHandler(index, dim, finalMatrix, fabricObject),
        actionHandler: anchorWrapper(index > 0 ? index - 1 : lastControl, actionHandler),
        actionName: "modifyPolygon",
        pointIndex: index
      });
      return acc;
    }, {});
    poly.hasBorders = false;
    canvas.requestRenderAll();
  };

  const addAndSetEventPolygon = points => {
    const options = {
      fill: canvasProps.canEdit === false ? "rgba(0,0,0,0)" : "rgba(208,201,203,0.5)",
      strokeWidth: 1,
      stroke: "green",
      scaleX: 1,
      scaleY: 1,
      objectCaching: false,
      transparentCorners: false,
      cornerColor: "blue",
      cornerStyle: "circle",
      cornerSize: 10,
      ...canvasProps.polygonStyle
    };
    const polygon = new fabric.Polygon(points, options);
    canvas.add(polygon);
    polygon.on("selected", editPolygon);
    polygon.on("mousedblclick", resizePolygon);
  };

  const handleAddPolygon = (type, size) => {
    if (canvas) {
      const points = createPolygon({
        xStart: canvasProps.width / 2,
        yStart: canvasProps.height / 2,
        type,
        size
      });
      addAndSetEventPolygon(points);
    }
  };

  const handleAddAndReplacePolygon = (id, points) => {
    if (canvas) {
      canvas.getObjects().forEach(item => {
        if (item.hasOwnProperty("id") && item.get("id") === id) {
          canvas.remove(item);
        }
      });
      const polygon = new fabric.Polygon(points, {
        fill: "rgba(0,0,0,0)",
        strokeWidth: 1,
        stroke: "red",
        scaleX: 1,
        scaleY: 1,
        objectCaching: false,
        transparentCorners: false,
        id
      });
      canvas.add(polygon);
      polygon.on("selected", editPolygon);
      polygon.on("mousedblclick", resizePolygon);
      canvas.requestRenderAll();
    }
  };

  const handleConfirmUpdate = () => {
    const polygonList = [];

    if (canvas) {
      for (let i = 0; i < canvas._objects.length; i++) {
        const {
          oCoords
        } = canvas._objects[i];
        const polygon = [];
        Object.keys(oCoords).forEach(key => {
          if (key.charAt(0) === "p") {
            polygon.push({
              x: oCoords[key].x < 0 ? 0 : oCoords[key].x,
              y: oCoords[key].y < 0 ? 0 : oCoords[key].y
            });
          }
        });

        if (polygon.length > 0) {
          polygonList.push({
            polygon,
            label: canvas._objects[i].label ? canvas._objects[i].label : `Label ${i + 1}`
          });
        } else {
          const {
            points
          } = canvas._objects[i];

          if (points.length > 0) {
            polygonList.push({
              polygon: points,
              label: canvas._objects[i].label ? canvas._objects[i].label : `Label ${i + 1}`
            });
          }
        }
      }

      canvas.requestRenderAll();
    }

    return polygonList;
  };

  const handleDeletePolygon = () => {
    if (canvas) {
      const activeObject = canvas.getActiveObject();
      canvas.remove(activeObject);
      canvas.requestRenderAll();
    }
  };

  const handleNextPolygon = () => {
    if (canvas) {
      let activeObject = canvas.getActiveObject();

      if (!activeObject) {
        activeObject = canvas.getObjects()[0];
        canvas.setActiveObject(activeObject);
      } else {
        for (let i = 0; i < canvas._objects.length; i++) {
          if (canvas._objects[i] === activeObject) {
            canvas.setActiveObject(i > canvas._objects.length - 2 ? canvas._objects[0] : canvas._objects[i + 1]);
            break;
          }
        }
      }
    }
  };

  const handlePreviousPolygon = () => {
    if (canvas) {
      let activeObject = canvas.getActiveObject();

      if (!activeObject) {
        activeObject = canvas.getObjects()[0];
        canvas.setActiveObject(activeObject);
      } else {
        for (let i = 0; i < canvas._objects.length; i++) {
          if (canvas._objects[i] === activeObject) {
            canvas.setActiveObject(i === 0 ? canvas._objects[canvas._objects.length - 1] : canvas._objects[i - 1]);
            break;
          }
        }
      }
    }
  };

  const handleMoving = obj => {
    obj.objectCaching = false;

    if (obj.currentHeight > obj.canvas.height || obj.currentWidth > obj.canvas.width) {
      return;
    }

    obj.setCoords();

    if (obj.getBoundingRect().top < 0 || obj.getBoundingRect().left < 0) {
      obj.top = Math.max(obj.top, obj.top - obj.getBoundingRect().top);
      obj.left = Math.max(obj.left, obj.left - obj.getBoundingRect().left);
    }

    if (obj.getBoundingRect().top + obj.getBoundingRect().height > obj.canvas.height || obj.getBoundingRect().left + obj.getBoundingRect().width > obj.canvas.width) {
      obj.top = Math.min(obj.top, obj.canvas.height - obj.getBoundingRect().height + obj.top - obj.getBoundingRect().top);
      obj.left = Math.min(obj.left, obj.canvas.width - obj.getBoundingRect().width + obj.left - obj.getBoundingRect().left);
    }

    canvas.requestRenderAll();
  };

  const toggleDrawPolygon = () => {
    if (drawMode) {
      activeLine = null;
      activeShape = null;
      lineArray = [];
      pointArray = [];
      canvas.selection = true;
      setDrawMode(false);
    } else {
      canvas.selection = false;
      setDrawMode(true);
    }
  };

  const addPoint = options => {
    const pointOption = {
      id: new Date().getTime(),
      radius: 5,
      fill: "#ffffff",
      stroke: "#333333",
      strokeWidth: 0.5,
      left: options.e.layerX / canvas.getZoom(),
      top: options.e.layerY / canvas.getZoom(),
      selectable: false,
      hasBorders: false,
      hasControls: false,
      originX: "center",
      originY: "center",
      objectCaching: false
    };
    const point = new fabric.Circle(pointOption);

    if (pointArray.length === 0) {
      point.set({
        fill: "red"
      });
    }

    const linePoints = [options.e.layerX / canvas.getZoom(), options.e.layerY / canvas.getZoom(), options.e.layerX / canvas.getZoom(), options.e.layerY / canvas.getZoom()];
    const lineOption = {
      strokeWidth: 2,
      fill: "#999999",
      stroke: "#999999",
      originX: "center",
      originY: "center",
      selectable: false,
      hasBorders: false,
      hasControls: false,
      evented: false,
      objectCaching: false
    };
    const line = new fabric.Line(linePoints, lineOption);
    line.class = "line";

    if (activeShape) {
      const pos = canvas.getPointer(options.e);
      const points = activeShape.get("points");
      points.push({
        x: pos.x,
        y: pos.y
      });
      const polygon = new fabric.Polygon(points, {
        stroke: "#333333",
        strokeWidth: 1,
        fill: "#cccccc",
        opacity: 0.3,
        selectable: false,
        hasBorders: false,
        hasControls: false,
        evented: false,
        objectCaching: false
      });
      canvas.remove(activeShape);
      canvas.add(polygon);
      activeShape = polygon;
      canvas.renderAll();
    } else {
      const polyPoint = [{
        x: options.e.layerX / canvas.getZoom(),
        y: options.e.layerY / canvas.getZoom()
      }];
      const polygon = new fabric.Polygon(polyPoint, {
        stroke: "#333333",
        strokeWidth: 1,
        fill: "#cccccc",
        opacity: 0.3,
        selectable: false,
        hasBorders: false,
        hasControls: false,
        evented: false,
        objectCaching: false
      });
      activeShape = polygon;
      canvas.add(polygon);
    }

    activeLine = line;
    pointArray.push(point);
    lineArray.push(line);
    canvas.add(line);
    canvas.add(point);
  };

  const generatePolygonFromFreeDraw = pointArray => {
    const points = [];

    for (const point of pointArray) {
      points.push({
        x: point.left,
        y: point.top
      });
      canvas.remove(point);
    }

    for (const line of lineArray) {
      canvas.remove(line);
    }

    canvas.remove(activeShape).remove(activeLine);
    addAndSetEventPolygon(points);
    toggleDrawPolygon();
  };

  const onMouseDown = options => {
    if (drawMode) {
      if (options.target && options.target.id === pointArray[0].id) {
        generatePolygonFromFreeDraw(pointArray);
      } else {
        addPoint(options);
      }
    }

    const evt = options.e;

    if (evt.altKey === true) {
      undefined.lastPosX = evt.clientX;
    }
  };

  const onMouseMove = options => {
    if (drawMode) {
      if (activeLine && activeLine.class === "line") {
        const pointer = canvas.getPointer(options.e);
        activeLine.set({
          x2: pointer.x,
          y2: pointer.y
        });
        const points = activeShape.get("points");
        points[pointArray.length] = {
          x: pointer.x,
          y: pointer.y
        };
        activeShape.set({
          points
        });
      }

      canvas.renderAll();
    }
  };

  const [outerCanvasContainer, setCanvasContainer] = React.useState(null);
  useEffect(() => {
    const newCanvas = new fabric.Canvas(id, {
      width: canvasProps.width,
      height: canvasProps.height
    });
    const tmp = document.getElementById(`${id}_container`);

    if (tmp) {
      setCanvasContainer(tmp);
      const ratio = newCanvas.getWidth() / newCanvas.getHeight();
      const containerWidth = tmp.clientWidth;
      const scale = containerWidth / newCanvas.getWidth();
      const zoom = newCanvas.getZoom() * scale;
      newCanvas.setDimensions({
        width: containerWidth,
        height: containerWidth / ratio
      });
      newCanvas.setViewportTransform([zoom, 0, 0, zoom, 0, 0]);
    }

    setCanvas(newCanvas);
    fabric.Image.fromURL(canvasProps.backgroundImage, myImg => {
      const img = myImg.set({
        left: 0,
        top: 0,
        width: newCanvas.width,
        height: newCanvas.height
      });
      newCanvas.setBackgroundImage(img, newCanvas.renderAll.bind(newCanvas), {
        scaleX: newCanvas.width / img.width,
        scaleY: newCanvas.height / img.height
      });
    });
  }, []);
  useEffect(() => {
    if (canvas) {
      new ResizeObserver(resizeCanvas).observe(outerCanvasContainer);

      if (canvasProps.defaultPolygons) {
        const polygons = canvasProps.defaultPolygons;

        for (let i = 0; i < polygons.length; i++) {
          const {
            polygon
          } = polygons[i];
          addAndSetEventPolygon(polygon);
        }
      }

      canvas.on("object:moving", e => handleMoving(e.target));
      canvas.on("object:modified", e => handleMoving(e.target));
    }
  }, [canvas]);
  useEffect(() => {
    if (canvas && drawMode) {
      canvas.off("mouse:down");
      canvas.off("mouse:move");
      canvas.on("mouse:down", onMouseDown);
      canvas.on("mouse:move", onMouseMove);
    }
  }, [drawMode]);
  useImperativeHandle(ref, () => ({
    toggleDraw() {
      toggleDrawPolygon();
    },

    onDraw(option) {
      handleAddPolygon(option.type, option.size);
    },

    onDrawAndReplace(option) {
      handleAddAndReplacePolygon(option.id, option.points);
    },

    onConfirm() {
      return handleConfirmUpdate();
    }

  }));

  const resizeCanvas = () => {
    if (outerCanvasContainer) {
      const ratio = canvas.getWidth() / canvas.getHeight();
      const containerWidth = outerCanvasContainer.clientWidth;
      const scale = containerWidth / canvas.getWidth();
      const zoom = canvas.getZoom() * scale;
      canvas.setDimensions({
        width: containerWidth,
        height: containerWidth / ratio
      });
      canvas.setViewportTransform([zoom, 0, 0, zoom, 0, 0]);
    }
  };

  return React.createElement("div", {
    id: `${id}_container`,
    style: {
      position: "relative"
    }
  }, React.createElement("div", {
    style: {
      position: "absolute",
      top: 0,
      left: 0,
      right: 0,
      bottom: 0
    }
  }, children), React.createElement("canvas", {
    id: id
  }), canvasProps.canEdit === false && React.createElement("div", {
    style: {
      position: "absolute",
      top: 0,
      left: 0,
      right: 0,
      bottom: 0
    }
  }));
});

const POLYGON_SIZE = {
  small: 50,
  normal: 100,
  large: 150
};
const POLYGON_TYPE = {
  rec: 4,
  hex: 6,
  oct: 8
};
const CanvasPolygons = forwardRef(({
  children,
  canvasHeight,
  canvasWidth,
  defaultPolygons,
  canEdit,
  polygonStyle
}, ref) => {
  const polygonRef = React.useRef();
  useImperativeHandle(ref, () => ({
    toggleDraw() {
      polygonRef.current.toggleDraw();
    },

    onDraw(option) {
      polygonRef.current.onDraw(option);
    },

    onDrawAndReplace(option) {
      polygonRef.current.onDrawAndReplace(option);
    },

    onConfirm() {
      return polygonRef.current.onConfirm();
    }

  }));
  return React.createElement(Polygon, {
    ref: polygonRef,
    canvasProps: {
      width: canvasWidth,
      height: canvasHeight,
      defaultPolygons,
      canEdit,
      polygonStyle
    }
  }, children);
});

export default CanvasPolygons;
export { POLYGON_SIZE, POLYGON_TYPE };
//# sourceMappingURL=index.modern.js.map
